// GPS code for Processing
// Glen Murphy, glen@glenmurphy.com
class P5gps {
	private boolean locked = false;
	private boolean available = false;
	private String latitude; // ddmm.mmmm
	private String longitude; // ddmm.mmmm
	private int north;
	private int east;
	private String knots;
	private String heading;
	private String altitude; // metres

	private String nmea;
	private String nmeaType;

	P5gps(int rate, String nmeaTypeIn) {
		beginSerial(rate);
		nmeaType = nmeaTypeIn;
	}

	void processNMEA(char nmeaIn) {
		nmea += nmeaIn;
		if(nmeaIn == '\n') {
			String dataBlock[] = splitStrings(nmea, ',');

			if(dataBlock[0].equals(nmeaType)) {
				if(nmeaType.equals("$GPRMC")) {
					latitude = dataBlock[3];
					north = (dataBlock[4].equals("N")) ? 1 : -1;

					longitude = dataBlock[5];
					east = (dataBlock[4].equals("E")) ? 1 : -1;

					knots = dataBlock[7];
					heading = dataBlock[8];

					locked = dataBlock[2].equals("A") ? true : false;
                                }
				else if (nmeaType.equals("$GPGGA")) {
					latitude = dataBlock[2];
					north = (dataBlock[3].equals("N")) ? 1 : -1;

					longitude = dataBlock[4];
					east = (dataBlock[5].equals("E")) ? 1 : -1;

					locked = (Integer.valueOf(dataBlock[6]).intValue() > 0) ? true : false;
					altitude = dataBlock[9];
				}
				available = true;
			}
			nmea = "";
		}
	}

	float minutesToDegrees(float dm) {
		return (int)(dm / 100) + (dm - ( (int)(dm/100) ) * 100) / 60;
	}

	float latitude() {
		if(!available) return 0;
		// convert from ddmm.mmmm to dd.dddddd
		return minutesToDegrees(Float.valueOf(latitude).floatValue()) * north;
	}

	float longitude() {
		if(!available) return 0;
		// convert from ddmm.mmmm to dd.dddddd
		return minutesToDegrees(Float.valueOf(longitude).floatValue()) * east;
	}

	float velocity() {
		if(!available) return 0;
		// return velocity in km/h
		return Float.valueOf(knots).floatValue() * 1.85;
	}

	float heading() {
		if(!available) return 0;
		return Float.valueOf(heading).floatValue();
	}

	boolean locked() {
		return locked;
	}

}
